// $Id: OopsMessageFilterList.m,v 1.1.1.1 2000/03/02 10:39:40 malte Exp $
//
// This file is part of the OopsMailFilter Bundle for Apple's MailViewer application.
// Copyright (c) 1999-2000 Oops AB. All rights reserved.
//
// The source file window should be wide enough to fit this paragraph. When using ProjectBuilder by
// Apple, switch on the option "Indent wrapped lines" and set the indentation to 4 spaces.
//
// See end of file for revision history.
//

#import "OopsMessageFilterList.h"
#import "OopsMessageFilter.h"

static NSDictionary *_defaultFileAttributes = nil;
static NSDictionary *_defaultDirectoryAttributes = nil;

@implementation OopsMessageFilterList

+ (void)initialize {
    if (self == [OopsMessageFilterList class]) {
        _defaultFileAttributes = [[NSDictionary dictionaryWithObject:[NSNumber numberWithUnsignedLong:(unsigned long)0644] forKey:NSFilePosixPermissions] retain];
        _defaultDirectoryAttributes = [[NSDictionary dictionaryWithObject:[NSNumber numberWithUnsignedLong:(unsigned long)0755] forKey:NSFilePosixPermissions] retain];
        /* Note: should apply the file creation mask of the current user. */
    }
}

+ (NSDictionary *)defaultFileAttributes {
    return _defaultFileAttributes;
}

+ (NSDictionary *)defaultDirectoryAttributes {
    return _defaultDirectoryAttributes;
}

+ (OopsMessageFilterList *)filterList {
	static OopsMessageFilterList *_messageFilterList = nil;

    if (!_messageFilterList) {
        _messageFilterList = [[OopsMessageFilterList alloc] init];
	}

    return _messageFilterList;
}

- (id)init {
	if (self = [super init]) {
        [self readFromFile];
	}
	return self;
}

- (void)dealloc {
	[filters release];
	[super dealloc];
}

- (void)setFilters:(NSArray *)someFilters {
	NSMutableArray *array = nil;

	if (!someFilters || ![someFilters count]) {
		return;
	}

	array = [[NSMutableArray alloc] initWithCapacity:[someFilters count]];
    [array addObjectsFromArray:someFilters];

	[filters release];
	filters = array;
}

- (NSMutableArray *)filters {
	return filters;
}

- (NSEnumerator *)filterEnumerator {
	NSEnumerator *arrayEnum = nil;
	NSMutableArray *array = nil;
	int count = [[self filters] count];

    array = [[NSMutableArray alloc] initWithCapacity:count];
	if (count) {
		[array addObjectsFromArray:[self filters]];
	}

	arrayEnum = [array objectEnumerator];
	[array autorelease];
	return arrayEnum;
}

- (NSString *)_getPath {
    NSString *path = NSHomeDirectory();
	BOOL isDir = NO;

    path = [path stringByAppendingPathComponent:@"Library"];
    path = [path stringByAppendingPathComponent:@"MailViewer"];
    path = [path stringByAppendingPathComponent:@"OopsMessageFilters.plist"];

	if (![[NSFileManager defaultManager] fileExistsAtPath:path isDirectory:&isDir]) {
        NSLog(@"%@: file at path %@ does not exist.", self, path);
		return nil;
	}

	if (isDir) {
        NSLog(@"%@: file at path %@ is a directory. Should be plain file.", self, path);
        return nil;
	}

	return path;
}

- (NSString *)_getPathForReading {
    NSFileManager *fmanager = [NSFileManager defaultManager];
	NSString *path = [self _getPath];

	if (!path) {
		return nil;
	}

	if (![fmanager isReadableFileAtPath:path]) {
		NSLog(@"%@: unable to read file at path %@.", self, path);
		return nil;
	}

	return path;
}

- (NSString *)_getPathForWriting {
	NSFileManager *fmanager = [NSFileManager defaultManager];
    NSString *path = [self _getPath];

    if (!path) {
        return nil;
    }

	if (![fmanager isDeletableFileAtPath:path]) {
        NSLog(@"%@: unable to write file at path %@.", self, path);
		return nil;
	}

	return path;
}

int orderFilters(id filter1, id filter2, void *nothing) {
    int order1, order2;

    order1 = [(OopsMessageFilter *)filter1 order];
    order2 = [(OopsMessageFilter *)filter2 order];

    if (order1 == order2) {
        return NSOrderedSame;
    } else if (order1 < order2) {
        return NSOrderedAscending;
    } else {
        return NSOrderedDescending;
    }
}

- (void)readFromFile {
    // Read contents of file ~/Library/MailViewer/OopsMessageFilters.plist.

	NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
	NSString *path = [self _getPathForReading];
	NSData *data = nil;
	NSString *string = nil;
	NSDictionary *dict = nil;
    NSArray *array = nil;
	int i, count;
	NSMutableArray *someFilters = nil;

	if (!path) {
		[pool release];
		return;
	}

	data = [[NSData alloc] initWithContentsOfFile:path];
    string = [[NSString alloc] initWithData:data encoding:NSASCIIStringEncoding];
    dict = [string propertyList];
	array = [dict objectForKey:OopsMessageFiltersKey];

	[data release];
	[string release];

    count = [array count];
	someFilters = [[NSMutableArray alloc] initWithCapacity:count];

	for (i=0; i<count; i++) {
		id aFilterDict = [array objectAtIndex:i];
		OopsMessageFilter *filter = [[OopsMessageFilter alloc] init];

		if ([filter loadFromPropertyListRepresentation:aFilterDict]) {
			[someFilters addObject:filter];
		}

		[filter release];
	}

    [someFilters sortUsingFunction:orderFilters context:nil];
	[self setFilters:someFilters];

	[someFilters release];
	[pool release];
}

- (void)writeToFile {
	NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
    NSString *path = [self _getPathForReading];
	NSMutableArray *array = nil;
	int i, count;
	NSMutableDictionary *dict = nil;
	NSData *data = nil;

	if (!path) {
		[pool release];
		return;
	}

	count = [[self filters] count];
	array = [[NSMutableArray alloc] initWithCapacity:count];

	for (i=0; i<count; i++) {
		OopsMessageFilter *filter = [[self filters] objectAtIndex:i];
        NSDictionary *dict = nil;

		[filter setOrder:i];
		if (dict = [filter propertyListRepresentation]) {
            [array addObject:dict];
		}
	}

	dict = [[NSMutableDictionary alloc] init];
	[dict setObject:array forKey:OopsMessageFiltersKey];
	[array release];

    data = [[dict description] dataUsingEncoding:NSASCIIStringEncoding];
	[dict release];

    if (![data writeToFile:path atomically:YES]) {
        NSLog(@"%@: Could not write to file %@.", self, path);
		// Fall through.
    }

	[pool release];
}

@end

NSString *OopsMessageFiltersKey = @"MessageFilters";

/* Revision history:
 * $Log: OopsMessageFilterList.m,v $
 * Revision 1.1.1.1  2000/03/02 10:39:40  malte
 * Initial release
 *
 */
