// $Id: OopsSimpleFilterRule.m,v 1.1.1.1 2000/03/02 10:39:40 malte Exp $
//
// This file is part of the OopsMailFilter Bundle for Apple's MailViewer application.
// Copyright (c) 1999-2000 Oops AB. All rights reserved.
//
// The source file window should be wide enough to fit this paragraph. When using ProjectBuilder by
// Apple, switch on the option "Indent wrapped lines" and set the indentation to 4 spaces.
//
// See end of file for revision history.
//

#import "OopsSimpleFilterRule.h"
#import "Message.h"
#import "MessageHeaders.h"

#import "NSDictionary-CaseAccesing.h"

@interface OopsSimpleFilterRule (PrivateSearching)
- (BOOL)_performSearchOnMessageBody:(Message *)message;
- (BOOL)_performSearchOnMessageHeaders:(Message *)message;
@end

@implementation OopsSimpleFilterRule

- (void)setHeaders:(NSArray *)array {
	[array retain];
	[headers release];
	headers = array;
}

- (NSArray *)headers {
	return headers;
}

- (void)setSearchOptions:(NSArray *)array {
	[array retain];
	[searchOptions release];
	searchOptions = array;
}

- (NSArray *)searchOptions {
	return searchOptions;
}

- (void)setSearchString:(NSString *)string {
	[string retain];
	[searchString release];
	searchString = string;
}

- (NSString *)searchString {
	return searchString;
}

- (BOOL)_searchOptionsContainsString:(NSString *)string {
    /* Using this method instead of just calling containsObject: on the array lets the user write the search option keys in any case. */
    NSArray *options = [self searchOptions];
    int i, count = [options count];

	if (!string) {
		return NO;
	}

    for (i=0; i<count; i++) {
        NSComparisonResult result = [[options objectAtIndex:i] caseInsensitiveCompare:string];

		if (result == NSOrderedSame) {
			return YES;
		}
    }

    return NO;
}

- (BOOL)isCaseSensitive {
	return [self _searchOptionsContainsString:@"casesensitive"];
}

- (BOOL)shouldContain {
    return [self _searchOptionsContainsString:@"contains"];
}

- (BOOL)shouldBeginWith {
    return [self _searchOptionsContainsString:@"begins"];
}

- (BOOL)shouldEndIn {
    return [self _searchOptionsContainsString:@"ends"];
}

- (BOOL)shouldBe {
    return [self _searchOptionsContainsString:@"is"];
}

- (BOOL)compliedToByMessage:(Message *)message {
	if (!message) {
		return NO;
	}

	if (!headers || ![headers count]) {
		return [self _performSearchOnMessageBody:message];
	} else {
		return [self _performSearchOnMessageHeaders:message];
	}

	// Never gets here.
	return NO;
}

- (BOOL)loadFromPropertyListRepresentation:(NSDictionary *)dict {
    id obj = nil;

    if (![super loadFromPropertyListRepresentation:dict]) {
        return NO;
    }

    obj = [dict objectForKey:OopsSimpleFilterRuleHeadersKey];
    if ([obj isKindOfClass:[NSArray class]]) {
        [self setHeaders:obj];
    }

    obj = [dict objectForKey:OopsSimpleFilterRuleSearchOptionsKey];
    if ([obj isKindOfClass:[NSArray class]]) {
        [self setSearchOptions:obj];
    }

    obj = [dict objectForKey:OopsSimpleFilterRuleSearchStringKey];
    if ([obj isKindOfClass:[NSString class]]) {
        [self setSearchString:obj];
    }

    return YES;
}

- (NSDictionary *)propertyListRepresentation {
    NSMutableDictionary *dict = nil;
    id obj = nil;

	dict = [super propertyListRepresentation];

    if (obj = [self headers]) {
        [dict setObject:obj forKey:OopsSimpleFilterRuleHeadersKey];
    }

    if (obj = [self searchOptions]) {
        [dict setObject:obj forKey:OopsSimpleFilterRuleSearchOptionsKey];
    }

    if (obj = [self searchString]) {
        [dict setObject:obj forKey:OopsSimpleFilterRuleSearchStringKey];
    }

    return dict;
}

@end

@implementation OopsSimpleFilterRule (PrivateSearching)

- (BOOL)_performSearchOnMessageBody:(Message *)message {
    NSLog(@"%@: Performing body search: %@", self, [self propertyListRepresentation]);
	return NO;
}

- (BOOL)_matchHeaderValue:(NSString *)value {

	if ([self shouldBe]) {
        /*
         * Does the string match exactely?
         */

		if ([self isCaseSensitive]) {
			return [value isEqualToString:[self searchString]];
		}

		if ([value caseInsensitiveCompare:[self searchString]] == NSOrderedSame) {
			return YES;
		}

		return NO;

	} else {
		/*
		 * Does the string match partially?
		 */

        NSRange range;
        BOOL exceptionRaised = NO;
        unsigned int options = 0;

        if ([self isCaseSensitive] == NO) {
            options = NSCaseInsensitiveSearch;
        }

		NS_DURING
			if (options) {
				range = [value rangeOfString:[self searchString] options:options];
			} else {
	            range = [value rangeOfString:[self searchString]];
			}

		NS_HANDLER
	        exceptionRaised = YES;
		NS_ENDHANDLER

        if (exceptionRaised) return NO;
        if (range.location == NSNotFound && range.length == 0) return NO;
        if (range.length == 0) return NO;

		if ([self shouldBeginWith]) {
			return (range.location == 0) ? YES : NO;

		} else if ([self shouldEndIn]) {
			return (NSMaxRange(range) == [value length]) ? YES : NO;

		} else if ([self shouldContain]) {
			// If we got here, value contained our search string.

			/*
			 * Fall through
			 */
		}

        return YES;
	}

	// Should never end up here.
	return NO;
}

- (BOOL)_performSearchOnMessageHeaders:(Message *)message {
    // NSLog(@"%@: Performing header search: %@", self, [self propertyListRepresentation]);

	/*
		1. Loop through header names.
			1.1	find a key in the message headers dict that match the header.
				1.1.1	Loop through contents of key.
					1.1.1.1	For each content, match against the searchString using
							specified search options.
				1.1.2	If any match, return YES, else NO.
			1.2 continue.
	*/

	NSArray *myHeaders = [self headers];
	NSEnumerator *myHeaderEnum = [myHeaders objectEnumerator];
	NSString *myHeader = nil;

	NSDictionary *messageHeaders = [[message headers] allHeaders];

	while (myHeader = [myHeaderEnum nextObject]) {
        NSArray *headerValues = [messageHeaders objectForCaseInsensitiveKey:myHeader];
		NSEnumerator *valueEnum = [headerValues objectEnumerator];
		NSString *aValue = nil;

		while (aValue = [valueEnum nextObject]) {
			if ([self _matchHeaderValue:aValue]) {
				// NSLog(@"%@: match on header %@", self, myHeader);
				return YES;
			}
		}
	}

	return NO;
}

@end

NSString *OopsSimpleFilterRuleHeadersKey = @"headers";
NSString *OopsSimpleFilterRuleSearchOptionsKey = @"searchOptions";
NSString *OopsSimpleFilterRuleSearchStringKey = @"searchString";

/* Revision history:
 * $Log: OopsSimpleFilterRule.m,v $
 * Revision 1.1.1.1  2000/03/02 10:39:40  malte
 * Initial release
 *
 */
